// Set up namespace and some state.
var mkdru = {
  // Settings to pass to pz2.js
  useSessions: Drupal.settings.mkdru.use_sessions === '1',
  //   showResponseType: 'json',
  // Variables
  active: false,
  pz2: null,
  totalRec: 0,
  pagerRange: 6,
  pazpar2Path: Drupal.settings.mkdru.pz2_path,
  // Facets
  facets: {
    source: {
      pz2Name: 'xtargets',
      max: Drupal.settings.mkdru.source_max
    },
    subject: {
      pz2Name: 'subject',
      max: Drupal.settings.mkdru.subject_max
    },
    author: {
      pz2Name: 'author',
      max: Drupal.settings.mkdru.author_max
    }
  },
  // State
  defaultState: {
    page: 1,
    perpage: 20,
    sort: 'relevance',
    filter: null,
    query:'',
    recid:null
  },
  state: {}
};



// So we can use jQuery BBQ with Drupal 6 and its 1.2.6 jQuery
if (!$.isArray) $.isArray = function(obj) {
  return Object.prototype.toString.call(obj) === "[object Array]";
};

// BBQ has no handy way to remove params without changing the hash.
// This takes an object to add and an array of keys to delete.
mkdru.hashAddDelMany = function (add, del) {
  var newHash = $.deparam.fragment();
  if (typeof(add) === 'object')
    $.extend(newHash, add);
  if ($.isArray(del))
    for (var i=0; i < del.length; i++)
      if (newHash[del[i]] !== 'undefined')
        delete newHash[del[i]];
  return $.param.fragment("#", newHash);
}

// It's sometimes cumbersome that object literals can't take variable keys.
mkdru.hashAddDelOne = function (key, value, del) {
  var toAdd;
  var toDel;
  if (key && value) {
    var toAdd = {};
    toAdd[key] = value;
  }
  if (del) {
    var toDel = [];
    toDel.push(del);
  }
  return mkdru.hashAddDelMany(toAdd, toDel);
}



// pz2.js event handlers:
mkdru.pz2Init = function () {
  if (mkdru.state.query) {
    mkdru.search();
  }
  mkdru.pz2.stat();
  //mkdru.pz2.bytarget();
};

mkdru.pz2Show = function (data) {
  mkdru.totalRec = data.merged;
  $('.mkdru-pager').html(mkdru.generatePager());
  $('.mkdru-counts').html(Drupal.theme('mkdruCounts', data.start + 1,
                                      data.num, data.merged, data.total));
  var html = "";
  for (var i = 0; i < data.hits.length; i++) {
    html += Drupal.theme('mkdruResult', data.hits[i], 
      i + 1 + mkdru.state.perpage * (mkdru.state.page - 1),
      "#" + $.param.fragment($.param.fragment(
        window.location.href, {recid: data.hits[i].recid})) + "\n"
    );
  }
  $('.mkdru-result-list').html(html);
  if (mkdru.state.recid) {
    mkdru.pz2.record(mkdru.state.recid);
  }
  else {
    $('.mkdru-results').show();
  }
};

mkdru.pz2Status = function (data) {
  $('.mkdru-status').html(Drupal.theme('mkdruStatus', data.activeclients, data.clients));
};

mkdru.pz2Term = function (data) {
  for (var facet in mkdru.facets) {
    // facet is limited
    if (mkdru.state['limit_' + facet]) {
      $('.mkdru-facet-' + facet).html(Drupal.theme('mkdruFacetLimit',
                                 data[mkdru.facets[facet].pz2Name][0],
                                 mkdru.hashAddDelOne(null, null, 'limit_' + facet)));
    } else {
      $('.mkdru-facet-' + facet).html(Drupal.theme('mkdruFacet',
                                 data[mkdru.facets[facet].pz2Name],
                                 facet, mkdru.facets[facet].max));
    }
  }
};

mkdru.pz2ByTarget = function (data) {
  
};

mkdru.pz2Record = function (data) {
  clearTimeout(mkdru.pz2.showTimer);
  $('.mkdru-results').hide();
  $('.mkdru-detail').html(Drupal.theme('mkdruDetail', data,
                                       mkdru.hashAddDelOne(null, null, 'recid')));
  $('.mkdru-detail').show();
  clearTimeout(mkdru.pz2.recordTimer);
};



// State and URL handling 

// populate state from an object and fill in the blanks with defaults
mkdru.stateFromObject = function (obj) {
  mkdru.state = $.extend({}, mkdru.defaultState);
  for (var key in mkdru.defaultState)
    if (typeof(obj[key]) != "undefined")
      mkdru.state[key] = obj[key];
};

// populate state from current window's hash string
mkdru.stateFromHash = function () {
  mkdru.stateFromObject($.deparam.fragment());
};

// set current window's hash string from state
mkdru.hashFromState = function () {
  // only include non-default settings in the URL
  var alteredState = {};
  for (var key in mkdru.defaultState) {
    if (mkdru.state[key] != mkdru.defaultState[key]) {
      alteredState[key] = mkdru.state[key];
    }
  }
  $.bbq.pushState(alteredState, 2);
};

// update mkdru_form theme's ui to match state
mkdru.uiFromState = function () {
  for (var key in mkdru.state) {
    switch(key) {
    case 'query':
      $('.mkdru-search input:text').attr('value', mkdru.state[key]);
      break;
    case 'perpage':
      $('.mkdru-perpage').attr('value', mkdru.state[key]);
      break;
    case 'sort':
      $('.mkdru-sort').attr('value', mkdru.state[key]);
      break;
    }
  }
};

mkdru.hashChange = function () {
  mkdru.stateFromHash();
  // Request for details
  if (mkdru.state.recid) {
    mkdru.pz2.record(mkdru.state.recid);
  }
  // Other internal link
  else {
    // may need to run search again to limit targets
    for (key in mkdru.state) {
      if (key.substring(0,5) === 'limit') {
        mkdru.search();
        break;
      }
    }
    mkdru.pz2.showPage(mkdru.state.page-1);
    $('.mkdru-detail').hide();
    $('.mkdru-results').show();
  }
};



//form submit handler
mkdru.submitQuery = function () {
  // new query, back to defaults (shallow copy)
  mkdru.state = $.extend({}, mkdru.defaultState);
  mkdru.state.query = $('.mkdru-search input:text').attr('value');
  mkdru.pollDropDowns();
  mkdru.hashFromState();
  mkdru.search();
  mkdru.active = true;
  return false;
};

//criteria drop-down (perpage, sort) handler
mkdru.submitCriteria = function () {
  mkdru.pollDropDowns();
  //search is not ON, do nothing
  if (!mkdru.active) return false;
  // pages mean different things now
  mkdru.state.page = 1;
  mkdru.hashFromState();
  mkdru.pz2.show(0, mkdru.state.perpage, mkdru.state.sort);
  return false;
}

mkdru.search = function () {
  var filter = null;
  var query = mkdru.state.query;

  // facet limit implementation
  if (mkdru.state.limit_source) {
    filter = 'pz:id=' + mkdru.state.limit_source;
  }
  if (mkdru.state.limit_subject) {
    query += ' and su="' + mkdru.state.limit_subject + '"';
  }
  if (mkdru.state.limit_author) {
    query += ' and au="' + mkdru.state.limit_author + '"';
  }
  mkdru.pz2.search(query, mkdru.state.perpage, mkdru.state.sort, filter);
  //inform others that the search is ON
  mkdru.active = true;
};

mkdru.pollDropDowns = function () {
  mkdru.state.perpage = $('.mkdru-perpage').attr('value');
  mkdru.state.sort = $('.mkdru-sort').attr('value');
};

mkdru.generatePager = function () {
 // make sure page param is a number, otherwise pageing frwd will be broken
  if (typeof mkdru.state.page == "string") {
    mkdru.state.page = Number(mkdru.state.page);
  }
  var total = Math.ceil(mkdru.totalRec / mkdru.state.perpage);
  var first = (mkdru.state.page - mkdru.pagerRange > 0)
      ? mkdru.state.page - mkdru.pagerRange : 1;
  var last = first + 2 * mkdru.pagerRange < total
      ? first + 2 * mkdru.pagerRange : total;  
  var prev = null;
  var next = null;
  var pages = [];

  if ((mkdru.state.page - 1) >= first) {
    prev = "#" + $.param.fragment($.param.fragment(
               window.location.href, {page: mkdru.state.page - 1}))
  }
  if ((mkdru.state.page + 1) <= total) {
    next = "#" + $.param.fragment($.param.fragment(
               window.location.href, {page: mkdru.state.page + 1}))
  }

  for (var i = first; i <= last; i++) {
    pages.push("#" + $.param.fragment($.param.fragment(
               window.location.href, {page: i})));
  }

  return Drupal.theme('mkdruPager', pages, first, mkdru.state.page,
                      total, prev, next);
};



// wait until the DOM is ready, bind events
// and instantiate pz2 library
$(document).ready(function () {
  $(window).bind( 'hashchange', mkdru.hashChange);
  $('.mkdru-search').bind('submit', mkdru.submitQuery);
  $('.mkdru-search input:text').attr('value', '');
  $('.mkdru-perpage').bind('change', mkdru.submitCriteria);
  $('.mkdru-sort').bind('change', mkdru.submitCriteria);

  // generate termlist for pz2.js and populate facet limit state
  var termlist = [];
  for (var key in mkdru.facets) {
    termlist.push(mkdru.facets[key].pz2Name);
    mkdru.defaultState['limit_' + key] = null;
  }

  mkdru.pz2 = new pz2( { "onshow": mkdru.pz2Show,
              "showtime": 500, //each timer (show, stat, term, bytarget) can be specified this way
              "pazpar2path": mkdru.pazpar2path,
              "oninit": mkdru.pz2Init,
              "onstat": mkdru.pz2Status,
              "onterm": mkdru.pz2Term,
              "termlist": termlist.join(','),
              "onbytarget": mkdru.pz2ByTarget,
              "usesessions" : mkdru.useSessions,
              "showResponseType": mkdru.showResponseType,
              "onrecord": mkdru.pz2Record,
              "autoInit": false } );

  // initialise state to hash string or defaults
  mkdru.stateFromHash();
  // and update UI to match
  mkdru.uiFromState();

  // ting thing
  if (typeof(Drupal.settings.mkdru.query) !== "undefined") {
    mkdru.state.query = Drupal.settings.mkdru.query
  }

  if (mkdru.useSessions) {
    mkdru.pz2.init();
  }
  else if (mkdru.state.recid) {
    mkdru.pz2.record(mkdru.state.recid);
  }
  else if (mkdru.state.query) {
    mkdru.search();
  }
});
